@@ -1,7 +1,15 @@
 /* At least for the things in C */
 
-#include <R_ext/Boolean.h>
-#include <R_ext/RS.h>
+#include <R.h>
+/* -> Rconfig.h, but also Boolean.h RS.h */
+
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext ("cluster", String)
+#else
+#define _(String) (String)
+#endif
+
 
 /* in ./meet.f --- called from most source files: */
 /* NO LONGER:
@@ -75,9 +83,9 @@ int F77_NAME(dysta)(int *nn, int *jpp, double *x, double *dys, int *ndyst,
 
 /* --------- ./pam.c ------------------*/
 
-void bswap(int *kk, int *nn, int *nrepr, Rboolean med_given,
+void bswap(int kk, int nsam, int *nrepr, Rboolean med_given, int trace_lev,
 	   double *dysma, double *dysmb, double *beter,
-	   double *dys, double *sky, double *s, double *obj);
+	   double *dys, double *sky, double s, double *obj);
 
 void cstat(int *kk, int *nn, int *nsend, int *nrepr, Rboolean all_stats,
 	   double *radus, double *damer, double *ttd, double *separ, double *s,
