@@ -1,15 +1,15 @@
-C-- $Id: fanny.f,v 1.7 2002/09/09 08:47:51 maechler Exp $
+C-- $Id: fanny.f,v 1.10 2005/08/30 07:48:18 maechler Exp $
 C   program for Fuzzy cluster ANalysis
 C
       subroutine fanny(nn,jpp,kk,x,dss,jdyss,valmd,jtmd,ndyst,
      1	   nsend,nelem,negbr, syl,p,dp,pt,nfuzz,esp,ef,dvec,
-     2	   ttsyl,eda,edb,obj,ncluv,sylinf,eps)
+     2	   ttsyl,eda,edb,obj,ncluv,sylinf,r,tol,maxit)
 c Arguments
-      integer nn,jpp,kk, jdyss, ndyst
+      integer nn,jpp,kk, jdyss, ndyst, maxit
 C	  nn   = number of objects
 C	  jpp  = number of variables for clustering
 C	  kk   = number of clusters
-      double precision ttsyl, eda,edb, eps
+      double precision ttsyl, eda,edb, r, tol
       double precision x(nn,jpp),p(nn,kk),dp(nn,kk)
       double precision dss(nn*(nn-1)/2)
 
@@ -41,8 +41,8 @@ C
  130  l=l+1
       if(dss(l).gt.s)s=dss(l)
       if(l.lt.nhalf)go to 130
-      call fuzzy(nn,nhalf,p,dp,pt,dss,esp,ef,eda,edb,kk,obj,eps)
-      call caddy(nn,p,kk,ktrue,nfuzz,ncluv,pt,nelem)
+      call fuzzy(nn,nhalf,p,dp,pt,dss,esp,ef,eda,edb,kk,obj,r,tol,maxit)
+      call caddy(nn,p,kk,ktrue,nfuzz,ncluv,pt)
       if(2.le.ktrue .and. ktrue.lt.nn) then
 	 call fygur(ktrue,nn,kk,nhalf,ncluv,nsend,nelem,
      1	      negbr,syl,dvec,pt,ttsyl,dss,s,sylinf)
@@ -95,24 +95,24 @@ c VARs
       end
 C
 C
-      subroutine fuzzy(nn,hh,p,dp,pt,dss,esp,ef,eda,edb,k,obj,eps)
+      subroutine fuzzy(nn,hh, p, dp, pt, dss, esp,ef, eda,edb,
+     +     k, obj, r, tol, nit)
 
 c     Args
-      integer nn,hh, k
+      integer nn,hh, k, nit
       double precision p(nn,k),dp(nn,k),pt(k), dss(hh)
-      double precision esp(k),ef(k), eda,edb, obj(2), eps
-C     eps is the precision for the iterations
+      double precision esp(k),ef(k), eda,edb, obj(2), r, tol
+C     r	  is the exponent, strictly larger than 1.0 -- was fixed to 2
+C     tol is the precision for the iterations
+C     nit is the maximal number of iterations -- was fixed to 500
 
 C
 c     VARs
-      integer kaunt, j,j1,j2,l,lx,m, nd,ndk, nyt
-      double precision ann,crt,cryt,dt,ddd, r, reen, rvers,rkme, xx, zk
-C     r	  is the exponent, strictly larger than 1.0
-C     nyt is the maximal number of iterations
+      integer it, j,jm,l,lx,m, nd,ndk
+      double precision ann,crt,cryt,dt,ddd, reen, rvers,rkme, xx, zk
+
 C
-      r=2.0
-      rvers=1./r
-      nyt=500
+      rvers= 1./r
 C
 C     initial fuzzy clustering
 C
@@ -147,10 +147,10 @@ C
 	    esp(l)=esp(l)+p(m,l)
 	    do 80 j=1,nn
 	       if(j.ne.m) then
-		  j2=min0(m,j)
-		  j1=(j2-1)*nn-(j2*(j2+1))/2+max0(m,j)
-		  dp(m,l)=dp(m,l)+ p(j,l)*dss(j1)
-		  ef(l)	 =ef(l)	 + p(j,l)*p(m,l)*dss(j1)
+		  jm=min0(m,j)
+		  jm=(jm-1)*nn-(jm*(jm+1))/2+max0(m,j)
+		  dp(m,l)=dp(m,l)+ p(j,l)*dss(jm)
+		  ef(l)	 =ef(l)	 + p(j,l)*p(m,l)*dss(jm)
 	       endif
  80	    continue
  90	 continue
@@ -161,7 +161,7 @@ C
 C
 C     start of iterations
 C
-      kaunt=1
+      it=1
       m=0
 C
 C     the new membership coefficients of the objects are calculated,
@@ -184,16 +184,17 @@ C
 	 esp(l)=esp(l)+pt(l)-p(m,l)
 	 do 230 j=1,nn
 	    if(j.ne.m) then
-	       j2=min0(m,j)
-	       j1=(j2-1)*nn-(j2*(j2+1))/2+max0(m,j)
-	       ddd=(pt(l)-p(m,l))*dss(j1)
+	       jm=min0(m,j)
+	       jm=(jm-1)*nn-(jm*(jm+1))/2+max0(m,j)
+	       ddd=(pt(l)-p(m,l))*dss(jm)
 	       dp(j,l)= dp(j,l)+ ddd
 	       ef(l)  = ef(l)  + 2.*p(j,l)*ddd
 	    endif
  230	 continue
 	 p(m,l)=pt(l)
  240  continue
-      if(m.lt.nn)go to 200
+      if(m.lt.nn) go to 200
+
       cryt=0.
       eda=0.
       do 250 l=1,k
@@ -202,19 +203,23 @@ C
 	 cryt=cryt+ef(l)/(esp(l)*2.)
  250  continue
 C
-C     criterion is printed and tested for convergence
+C     Convergence check
 C
-      if((crt/cryt-1.).le.eps)go to 500
-      if(kaunt.lt.nyt)go to 300
-      go to 500
- 300  m=0
-      kaunt=kaunt+1
+      if(dabs(cryt - crt) .le. tol*cryt) go to 500
+      if(it .ge. nit) then
+c        non-convergence
+         nit = -1
+         go to 500
+      endif
+
+      m=0
+      it=it+1
       crt=cryt
       go to 200
 C
 C     non-fuzzyness index of libert is computed
 C
- 500  obj(1)=kaunt
+ 500  obj(1)=it
       obj(2)=cryt
       zk=k
       edb=(zk*eda-1.)/(zk-1.)
@@ -227,11 +232,11 @@ C
       end
 C
 C
-      subroutine caddy(nn,p,k,ktrue,nfuzz,ncluv,rdraw,nelem)
+      subroutine caddy(nn,p,k,ktrue,nfuzz,ncluv,rdraw)
 
 c     Args
       integer nn, k, ktrue
-      integer nfuzz(k), ncluv(nn), nelem(nn)
+      integer nfuzz(k), ncluv(nn)
       double precision p(nn,k), rdraw(k)
 C
 c     VARs
@@ -243,9 +248,9 @@ c     VARs
       nbest=1
       do 10 l=2,k
 	 if(pbest .lt. p(1,l)) then
-            pbest=p(1,l)
-            nbest=l
-         endif
+	    pbest=p(1,l)
+	    nbest=l
+	 endif
  10   continue
       nfuzz(1)=nbest
       ncluv(1)=1
@@ -254,37 +259,37 @@ c     VARs
 	 pbest=p(m,1)
 	 nbest=1
 	 do 30 l=2,k
-            if(pbest .lt. p(m,l)) then
-               pbest=p(m,l)
-               nbest=l
-            endif
+	    if(pbest .lt. p(m,l)) then
+	       pbest=p(m,l)
+	       nbest=l
+	    endif
  30	 continue
 	 stay= .FALSE.
 	 do 40 ktry=1,ktrue
 	    if(nfuzz(ktry).eq. nbest) then
-               stay=.TRUE.
-               ncluv(m)=ktry
-            endif
+	       stay=.TRUE.
+	       ncluv(m)=ktry
+	    endif
  40	 continue
 	 if(.not. stay) then
-            ktrue=ktrue+1
-            nfuzz(ktrue)=nbest
-            ncluv(m)=ktrue
-         endif
+	    ktrue=ktrue+1
+	    nfuzz(ktrue)=nbest
+	    ncluv(m)=ktrue
+	 endif
  20   continue
       if(ktrue .lt. k) then
-         do 60 kwalk=ktrue+1, k
-            do 70 kleft=1,k
-               stay=.FALSE.
-               do 80 ktry=1,kwalk-1
-                  if(nfuzz(ktry).eq.kleft) stay=.TRUE.
- 80            continue
-               if(.not. stay) then
-                  nfuzz(kwalk)=kleft
-                  go to 60
-               endif
- 70         continue
- 60      continue
+	 do 60 kwalk=ktrue+1, k
+	    do 70 kleft=1,k
+	       stay=.FALSE.
+	       do 80 ktry=1,kwalk-1
+		  if(nfuzz(ktry).eq.kleft) stay=.TRUE.
+ 80	       continue
+	       if(.not. stay) then
+		  nfuzz(kwalk)=kleft
+		  go to 60
+	       endif
+ 70	    continue
+ 60	 continue
       endif
 
       do 110 m=1,nn
@@ -297,24 +302,23 @@ c     VARs
  130	 continue
  110  continue
       end
-c----------------------------------------------------------------------------
+c     -----------------------------------------------------------
 c
-c Compute Silhouette Information :
+c     Compute Silhouette Information :
 c
-c CLEANUP: this is almost identical to  black() in  pam.f()
+c CLEANUP: this is almost identical to	black() in  pam.f()
 c   -- only difference : different  dys() indexing !
 c
       subroutine fygur(ktrue,nn,kk,hh,ncluv,nsend,nelem,
      1	   negbr,syl,srank,avsyl,ttsyl,dss,s,sylinf)
-c     Args
+c Args
       integer ktrue,nn,kk,hh
       integer ncluv(nn), nsend(nn), nelem(nn), negbr(nn)
       double precision syl(nn),srank(nn),avsyl(kk),ttsyl,dss(hh)
       double precision s, sylinf(nn,4)
-c     VARs
-      integer j,l, lang, lplac, mjl
-      integer nsylr, ntt, nj, numcl, nclu, nbb, nl, njl
-      double precision db,dysa,dysb, att,btt, rtt, rnn, symax
+c VARs
+      integer j,l, lang,lplac, nsylr, nclu,numcl,ntt,nj,njl,nl,nbb,mjl
+      double precision dysa,dysb,db, btt,rtt, symax
 
       nsylr=0
       ttsyl=0.0
@@ -343,6 +347,7 @@ c     VARs
 			else if(l.gt.nj) then
 			   mjl=nn*(nj-1)+l-nj*(nj+1)/2
 			   db=db+dss(mjl)
+C			else dss(.)=0 ; nothing to add
 			endif
 		     endif
  43		  continue
@@ -354,40 +359,52 @@ c     VARs
 		  endif
 	       endif
  41	    continue
-	    if(ntt.eq.1)go to 50
-	    dysa=0.0
-	    do 45 l=1,ntt
-	       nl=nelem(l)
-	       if(nj.lt.nl) then
-		  njl=nn*(nj-1)+nl-nj*(nj+1)/2
-		  dysa=dysa+dss(njl)
-	       else if(nj.gt.nl) then
-		  njl=nn*(nl-1)+nj-nl*(nl+1)/2
-		  dysa=dysa+dss(njl)
+
+	    if(ntt.gt.1) then
+	       dysa=0.0
+	       do 45 l=1,ntt
+		  nl=nelem(l)
+		  if(nj.lt.nl) then
+		     njl=nn*(nj-1)+nl-nj*(nj+1)/2
+		     dysa=dysa+dss(njl)
+		  else if(nj.gt.nl) then
+		     njl=nn*(nl-1)+nj-nl*(nl+1)/2
+		     dysa=dysa+dss(njl)
+C		  else dss(.)=0 ; nothing to add
+		  endif
+ 45	       continue
+	       dysa=dysa/(ntt - 1)
+	       if(dysa.gt.0.0)then
+		  if(dysb.gt.0.0) then
+		     if(dysb.gt.dysa) then
+			syl(j)=1.0-dysa/dysb
+		     else if(dysb.lt.dysa) then
+			syl(j)=dysb/dysa-1.0
+		     else
+c     dysb == dysa:
+			syl(j)=0.0
+		     endif
+		     if(syl(j).le. -1.0) syl(j)=-1.0
+		     if(syl(j).ge.  1.0) syl(j)= 1.0
+		  else
+		     syl(j)=-1.0
+		  endif
+	       else if(dysb.gt.0.0) then
+		  syl(j)=1.0
+	       else
+		  syl(j)=0.0
 	       endif
- 45	    continue
-	    att=ntt-1
-	    dysa=dysa/att
-	    if(dysa.gt.0.0)go to 51
-	    if(dysb.gt.0.0)go to 52
- 50	    syl(j)=0.0
-	    go to 40
- 52	    syl(j)=1.0
-	    go to 40
- 51	    if(dysb.le.0.0)go to 53
-	    if(dysb.gt.dysa)syl(j)=1.0-dysa/dysb
-	    if(dysb.lt.dysa)syl(j)=dysb/dysa-1.0
-	    if(dysb.eq.dysa)syl(j)=0.0
-	    go to 54
- 53	    syl(j)=-1.0
- 54	    if(syl(j).le.(-1.0))syl(j)=-1.0
-	    if(syl(j).ge.1.0)syl(j)=1.0
+	    else
+c     ntt == 1:
+	       syl(j)=0.0
+	    endif
  40	 continue
+
 	 avsyl(numcl)=0.0
 	 do 60 j=1,ntt
 	    symax=-2.0
 	    do 70 l=1,ntt
-	       if(symax.lt. syl(l)) then
+	       if(symax .lt. syl(l)) then
 		  symax=syl(l)
 		  lang=l
 	       endif
@@ -408,7 +425,7 @@ c     VARs
 	    sylinf(nsylr,3)=0.0
 	    sylinf(nsylr,4)=nelem(1)
 	 else
- 75	    do 80 l=1,ntt
+	    do 80 l=1,ntt
 	       nsylr=nsylr+1
 	       lplac=nsend(l)
 	       sylinf(nsylr,1)=numcl
@@ -418,6 +435,5 @@ c     VARs
  80	    continue
 	 endif
  100  continue
-      rnn=nn
-      ttsyl=ttsyl/rnn
+      ttsyl=ttsyl/nn
       end
